import 'dart:io';
import 'package:wrapper_args/wrapper_args.dart';
import '../utils/version.dart';
import './go/export.dart';

Command init() {
  final parser = ArgParser()
    ..addFlag(
      "help",
      abbr: "h",
      negatable: false,
      help: "Print this usage information",
    );
  return Command(
    name: "go",
    help: "generate go code",
    longHelp: "Usage: generate go [<subcommand>] [<options>]",
    parser: parser,
    execute: (command, results) {
      if (results["help"]) {
        print(command.usage);
      }
    },
  )..addCommand(
      [
        _init(),
        _version(),
      ],
    );
}

Command _version() {
  final parser = ArgParser()
    ..addFlag(
      "help",
      abbr: "h",
      negatable: false,
      help: "Print this usage information",
    )
    ..addOption(
      "control",
      defaultsTo: Version.defaultsTo,
      allowed: Version.allowed,
      help: "version control",
    )
    ..addOption(
      "output",
      abbr: "o",
      defaultsTo: "version/version.go",
    );
  return Command(
    name: "version",
    help: "generate go version code from source",
    longHelp: "Usage: generate go version [<options>]",
    parser: parser,
    execute: (command, results) {
      if (results["help"]) {
        print("${command.usage}");
        return 0;
      }
      () async {
        final stat = await FileStat.stat("go.mod");
        if (stat.type == FileSystemEntityType.notFound) {
          throw StateError("go.mod not found");
        }
        final output = results["output"];
        final version = Version.fromControl(results["control"]);

        print('''
tag      = ${version.tag} 
commit   = ${version.commit}
datetime = ${version.datetime}
''');

        var file = File(output);
        file = await file.create(recursive: false);
        await file.writeAsString('''
package version

// Tag .
const Tag = "${version.tag}";
// Commit .
const Commit = "${version.commit}";
// Date .
const Date = "${version.datetime}";
''');
      }();
    },
  );
}

Command _init() {
  final parser = ArgParser()
    ..addFlag(
      "help",
      abbr: "h",
      negatable: false,
      help: "Print this usage information",
    )
    ..addOption(
      "template",
      defaultsTo: "console",
      allowed: ["console"],
      allowedHelp: {
        "console": "A command-line application sample",
      },
      help: "project template",
    )
    ..addOption(
      "package-name",
      help: "package name",
    )
    ..addOption(
      "project-name",
      help: "package name",
    );
  return Command(
    name: "init",
    help: "init go project at this directory",
    longHelp: "Usage: generate go init [<options>]",
    parser: parser,
    execute: (command, results) {
      if (results["help"]) {
        print("${command.usage}");
        return 0;
      }
      String packageName = results["package-name"] ?? "";
      if (packageName.isEmpty) {
        throw ArgumentError("--package-name not specified");
      }
      packageName = formatPackage(packageName);
      String projectName =
          results["project-name"] ?? packageToProject(packageName);
      if (projectName.isEmpty) {
        throw ArgumentError("--project-name not specified");
      }
      projectName = formatProject(projectName);
      final version = Version.fromControl(Version.defaultsTo);
      final goConsole = GoConsole(
        packageName: packageName,
        projectName: projectName,
        version: version,
      );

      goConsole.done();
    },
  );
}
